/* gtkGL
 * Copyright (C) 1997,1998 AOSASA Shigeru
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "gtkGLmain.h"
#include "gtkGLcontext.h"
#include "gtkGLdebug.h"


static const gchar *gtk_gl_context_key = "gtk-gl-context";
static guint        gtk_gl_context_key_id = 0;


void
gtk_gl_init (int    *argc,
	     char ***argv)
{
  gdk_gl_init (argc, argv);

#ifdef G_ENABLE_DEBUG
  gtk_gl_debug_flags_init (argc, argv);
#endif /* G_ENABLE_DEBUG */
}

void
gtk_gl_adapt_widget (GtkWidget *widget)
{
  GtkGLContext *context;

  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_WIDGET (widget));
  g_return_if_fail (gtk_gl_context_of_widget (widget) == NULL);

  context = (GtkGLContext *)gtk_gl_context_new ();
  gtk_gl_context_set_owner (context, widget);

  if (!gtk_gl_context_key_id)
    gtk_gl_context_key_id = gtk_object_data_force_id (gtk_gl_context_key);

  gtk_object_set_data_by_id_full (GTK_OBJECT (widget),
				  gtk_gl_context_key_id, context,
				  (GtkDestroyNotify)gtk_object_unref);
}

GtkGLContext*
gtk_gl_context_of_widget (GtkWidget *widget)
{
  GtkGLContext *context;
    
  g_return_val_if_fail (widget != NULL,         NULL);
  g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);

  if (!gtk_gl_context_key_id)
    gtk_gl_context_key_id = gtk_object_data_force_id (gtk_gl_context_key);

  context = gtk_object_get_data_by_id (GTK_OBJECT (widget),
				       gtk_gl_context_key_id);

  return context;
}

void
gtk_gl_swap_buffers_of_widget (GtkWidget *widget)
{
  GtkGLContext *context;

  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_WIDGET (widget));
  g_return_if_fail (GTK_WIDGET_REALIZED (widget));

  context = gtk_gl_context_of_widget (widget);

/*
    ** CRITICAL **: file gtkGLmain.c: 
	line 88 (gtk_gl_swap_buffers_of_widget): 
	assertion `context != NULL' failed.
*/

  g_return_if_fail (context != NULL);
  g_return_if_fail (context->owner_widget == widget);


  gdk_gl_swap_buffers (widget->window);
}
