/* gtkGL
 * Copyright (C) 1997,1998 AOSASA Shigeru
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "gtkGLutils.h"


static GtkGLContext* gtk_gl_get_top_context    (void);
static GtkGLContext* gtk_gl_get_second_context (void);


static GSList *gtk_gl_context_stack = NULL;


void
gtk_gl_push_context (GtkGLContext *context)
{
  g_return_if_fail (context != NULL);

  gtk_gl_context_stack
      = g_slist_prepend(gtk_gl_context_stack, context);
}

void
gtk_gl_pop_context (void)
{
  GSList *tmp;

  if (gtk_gl_context_stack != NULL)
    {
      tmp = gtk_gl_context_stack;
      gtk_gl_context_stack = tmp->next;
      g_slist_free_1(tmp);
    }
}

static GtkGLContext*
gtk_gl_get_top_context (void)
{
  GSList *tmp;
  GtkGLContext *top_context;

  tmp = g_slist_nth (gtk_gl_context_stack, 0);
  if (tmp != NULL)
    top_context = (GtkGLContext *)tmp->data;
  else
    top_context = NULL;

  return top_context;
}

static GtkGLContext*
gtk_gl_get_second_context (void)
{
  GSList *tmp;
  GtkGLContext *second_context;

  tmp = g_slist_nth (gtk_gl_context_stack, 1);
  if (tmp != NULL)
    second_context = (GtkGLContext *)tmp->data;
  else
    second_context = NULL;

  return second_context;
}


gint
gtk_gl_wrap_begin (GtkGLContext *context,
		   gint          nest)
{
  gint r;

  g_return_val_if_fail (context != NULL,             FALSE);
  g_return_val_if_fail (GTK_IS_GL_CONTEXT (context), FALSE);
  if (nest == FALSE)
    g_return_val_if_fail (gtk_gl_get_top_context () == NULL, FALSE);
  else
    g_return_val_if_fail (gtk_gl_get_top_context () != NULL, FALSE);


  gtk_gl_push_context (context);
  r = gtk_gl_set_active_context (context);

  return r;
}

gint
gtk_gl_wrap_end (GtkGLContext *context,
		 gint          nest)
{
  gint r;

  if (nest == FALSE)
    g_return_val_if_fail (gtk_gl_get_second_context () == NULL, FALSE);
  else
    g_return_val_if_fail (gtk_gl_get_second_context () != NULL, FALSE);


  gtk_gl_pop_context ();

  if (nest == FALSE)
    {
      gtk_gl_unset_active_context ();
      r = TRUE;
    }
  else
    {
      r = gtk_gl_set_active_context (gtk_gl_get_top_context ());
    }

  return r;
}

gint
gtk_gl_wrap_begin_strict (GtkGLContext *context,
			  gint          nest)
{
  return gtk_gl_wrap_begin (context, nest);
}

gint
gtk_gl_wrap_end_strict (GtkGLContext *context,
			gint          nest)
{
  g_return_val_if_fail (context != NULL,                      FALSE);
  g_return_val_if_fail (GTK_IS_GL_CONTEXT (context),          FALSE);
  g_return_val_if_fail (context == gtk_gl_get_top_context (), FALSE);

  return gtk_gl_wrap_end (context, nest);
}

gint
gtk_gl_wrap_begin_defaults (gint nest)
{
  GtkGLContext *context;
  gint r;

  if (nest == FALSE)
    g_return_val_if_fail (gtk_gl_get_second_context () == NULL, FALSE);
  else
    g_return_val_if_fail (gtk_gl_get_second_context () != NULL, FALSE);


  context = gtk_gl_get_top_context ();
  r = gtk_gl_set_active_context (context);

  return r;
}

gint
gtk_gl_wrap_end_defaults (gint nest)
{
  if (nest == FALSE)
    g_return_val_if_fail (gtk_gl_get_second_context () == NULL, FALSE);
  else
    g_return_val_if_fail (gtk_gl_get_second_context () != NULL, FALSE);


  gtk_gl_unset_active_context ();

  return TRUE;
}
